#!/usr/bin/env python

# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import sys, os
import numpy as np
import pylab as plt

import ac_settings as ac_set
import ac_tools


# IMPORTANT: all indices names must start with "I_" in config file
def plt_time(info, out_dir, save_plt=False, rmv_flgs=False):
    """
    Saves timeseries plots of the indices identified in the rdb file by
    starting with 'I_'.
    """

    if out_dir is False: return

    print()
    print("Executing plt_time:")

    obj = info['obj']
    file_type = info['file_type']
    instr = info['instr']

    rdb_name = "{}_{}_{}_{}".format(obj, instr, file_type, ac_set.fnames['data'])
    rdb_file = os.path.join(out_dir, obj, rdb_name)

    width, height = ac_tools.plot_params()

    data = ac_tools.read_rdb(rdb_file)[0]

    bjd_raw = np.asarray(data['bjd'])
    bjd = bjd_raw - int(min(bjd_raw))

    if len(bjd) == 1:
        print("Only one data point, no need to plot.")
        return

    data_keys = list(data)

    ind_ids = []
    for k in range(len(data_keys)):
        if any(map(data_keys[k].startswith, 'I_')):
            ind_ids.append(('_').join(data_keys[k].split('_')[:2]))

    if not ind_ids:
        print("*** ERROR: No indices detected in file:")
        print("***", rdb_file)
        return

    # remove duplicates of ind_id and gives a list of selected indices
    ind_ids = list(set(ind_ids))

    # make new dictionary just for plots
    ind = {}
    for k in range(len(ind_ids)):
        ind[ind_ids[k]] = data[ind_ids[k]]
        ind_err_key = '{}_err'.format(ind_ids[k])
        ind_flg_key = '{}_flg'.format(ind_ids[k])
        ind[ind_err_key] = data[ind_err_key]
        ind[ind_flg_key] = data[ind_flg_key]

    for k in range(len(ind_ids)):
        plt.figure(figsize=(width,height))

        N = 0
        for i in range(len(ind[ind_ids[k]])):
            ind_flg = ind['{}_flg'.format(ind_ids[k])][i]
            ind_err = ind['{}_err'.format(ind_ids[k])][i]
            ind_val = ind[ind_ids[k]][i]
            if ind_flg == 'None':
                N += 1
                plt.errorbar(bjd[i], ind_val, ind_err, c='k', marker='.', ls='')
            if ind_flg == 'negFlux':
                if rmv_flgs is False:
                    N += 1
                    plt.errorbar(bjd[i], ind_val, ind_err, c='r', marker='.', ls='')
                else: pass

        plt.annotate(obj, xy=(0.05,0.9), xycoords='axes fraction', textcoords='axes fraction')

        plt.annotate("N = {}".format(N), xy=(0.05,0.85), xycoords='axes fraction', textcoords='axes fraction')

        plt.xlabel('BJD - {} [days]'.format(int(min(bjd_raw))))
        plt.ylabel(ind_ids[k])

        if save_plt is True:
            save_name = '{}_{}_{}_{}_{}'.format(obj, instr, file_type, ind_ids[k], ac_set.fnames['time_plt'])
            save_file = os.path.join(out_dir, obj, save_name)
            plt.savefig(save_file)
            print("{} timeseries saved to:\n{}".format(ind_ids[k], save_file))

        plt.close()

    return


def plt_time_mlty(info, out_dir, save_plt=False, rmv_flgs=False, hdrs=['I_CaII', 'I_Ha', 'I_NaI', 'I_HeI']):
    """
    Saves timeseries plots of the indices identified in the rdb file by
    starting with 'I_' in a 'multi-plot' format.
    """

    if not out_dir: return

    print()
    print("Executing plt_time_mlty:")

    if not info:
        print("*** ERROR: There is no 'info' dictionary.")
        return
    if not hdrs:
        print("*** ERROR: No input indices. Use '-i' and insert index names that are listed in the config file. To know location of config file call ACTIN without any arguments. To read a config file from a different location use '-cf' and add the path/file.")
        return
    if len(hdrs) == 1:
        print("Only one index selected, no need for multiplot")
        return

    obj = info['obj']
    file_type = info['file_type']
    instr = info['instr']

    rdb_name = "{}_{}_{}_{}".format(obj, instr, file_type, ac_set.fnames['data'])
    rdb_file = os.path.join(out_dir, obj, rdb_name)

    data = ac_tools.read_rdb(rdb_file)[0]

    bjd_raw = np.asarray(data['bjd'])
    bjd = bjd_raw - int(min(bjd_raw))

    if len(bjd) == 1:
        print("Only one data point, no need to plot.")
        return

    ind = {} # make new dictionary just for plots
    ind_ids = [] # only the indices in data
    for k in range(len(hdrs)):
        if hdrs[k] in list(data):
            ind[hdrs[k]] = data[hdrs[k]]
            ind_err_key = '{}_err'.format(hdrs[k])
            ind_flg_key = '{}_flg'.format(hdrs[k])
            ind[ind_err_key] = data[ind_err_key]
            ind[ind_flg_key] = data[ind_flg_key]
            ind_ids.append(hdrs[k])

    if len(ind_ids) == 1:
        print("Only one index in rdb file, no need for multiplot.")
        return

    if len(ind_ids) > 5:
        print("Too many indices for multiplot.")
        return

    width, height = ac_tools.plot_params(7, 4.5)
    height = 2*len(ind_ids)

    plot_n = 0
    n_subplots = len(ind_ids)
    plt.figure(figsize=(width, height))
    for k in range(len(ind_ids)):
        plot_n += 1
        ax1 = plt.subplot(n_subplots, 1, plot_n)

        N = 0
        for i in range(len(ind[ind_ids[k]])):
            ind_flg = ind['{}_flg'.format(ind_ids[k])][i]
            ind_val = ind[ind_ids[k]][i]
            ind_err = ind['{}_err'.format(ind_ids[k])][i]
            if ind_flg == 'None':
                N += 1
                plt.errorbar(bjd[i], ind_val, ind_err, c='k', marker='.',ls='')
            if ind_flg == 'negFlux':
                if rmv_flgs == False:
                    N += 1
                    plt.errorbar(bjd[i], ind_val, ind_err, c='r', marker='.',ls='')
                else: pass

        plt.xlabel('BJD - {} [days]'.format(int(min(bjd_raw))))
        plt.ylabel(ind_ids[k])

        from matplotlib.ticker import MaxNLocator
        ax1.yaxis.set_major_locator(MaxNLocator(prune='both'))

        if plot_n != len(ind_ids):
            frame1 = plt.gca()
            frame1.axes.get_xaxis().set_ticklabels([])

    n = len(ind_ids)
    plt.annotate(obj, xy=(0.05,1.*n+0.05), xycoords='axes fraction', textcoords='axes fraction')

    plt.annotate("N = {}".format(N), xy=(0.85,1.*n+0.05), xycoords='axes fraction', textcoords='axes fraction')

    plt.subplots_adjust(hspace=0.000)

    if save_plt == True:
        save_name =  '{}_{}_{}_{}_{}'.format(obj, instr, file_type, ('_').join(ind_ids), ac_set.fnames['time_mlty_plt'])
        save_file = os.path.join(out_dir, obj, save_name)
        plt.savefig(save_file)
        print("{} multiplot saved to:\n{}".format((', ').join(ind_ids), save_file))

    plt.close()

    return



#!/usr/bin/env python

# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import os, sys ### test
import numpy as np

# ACTIN FILES
import ac_settings as ac_set
import ac_get_win


def check_lines(wave, sel_lines):
    """
    Tests if the selected lines from config file fit inside the spectrograph
    wavelength range and fit inside any spectral orders for the case of 2d
    spectrum.
    """

    print("\nCHECKING LINES FOR WAVELENGTH RANGE AND SP. ORDERS")
    print("--------------------------------------------------")

    if type(wave[0]) is np.ndarray: # 2d spec
        min_spec_wave = wave[0][0]
        max_spec_wave = wave[-1][-1]
        spec_type = '2d'
    if type(wave[0]) in [np.float, np.float64]: # 1d spec
        min_spec_wave = wave[0]
        max_spec_wave = wave[-1]
        spec_type = '1d'

    # For each row (sp. line) in the config table calculate the min and max values of bandwidth
    rows = len(sel_lines['ln_id'])
    for k in range(rows):
        ln_id = sel_lines['ln_id'][k]
        ln_ctr = sel_lines['ln_ctr'][k]
        ln_win = sel_lines['ln_win'][k]

        if ln_win <= 0:
            sys.exit("*** ERROR: line {} bandwidth is not positive.".format(ln_id))

        min_wave = ln_ctr - ln_win/2.
        max_wave = ln_ctr + ln_win/2.

        # Check if line fits inside spectral range
        if min_wave < min_spec_wave or max_wave > max_spec_wave:
            sys.exit("*** ERROR: Line {} bandwidth outside spectral range.".format(ln_id))
        else: print("Line {} inside spectral range".format(ln_id))

        # If wave is 2d check if line fits inside sp. order
        if spec_type == '2d':
            order = None
            ln_ctr_orders = []
            order = []
            for i in range(len(wave)):
                if min_wave > wave[i][0] and max_wave < wave[i][-1]:
                    order.append(i)
                # used to show potencial orders with wavelength range
                if ln_ctr > wave[i][0] and ln_ctr < wave[i][-1]:
                    ln_ctr_orders.append(i)

            if order is None:
                print("*** ERROR: Could not determine sp. order for {}".format(ln_id))
                print("\tmin_wave = {:.2f}".format(min_wave))
                print("\tmax_wave = {:.2f}".format(max_wave))
                print("\tThe closest orders are:")
                for k in range(len(ln_ctr_orders)):
                    closest_ord = ln_ctr_orders[k]
                    print("\tOrder {}: {:.2f}-{:.2f}".format(closest_ord, wave[closest_ord][0], wave[closest_ord][-1]))
                sys.exit()
            else:
                for i in range(len(order)):
                    print("Line {} inside spectral order {}".format(ln_id, order[i]))
    return


def calc_flux_lines(data, sel_lines, ln_plts=False, frac=True):
    """
    Calculates the sum of the flux and associated errors for all spectral lines required to calculate the selected indices.

    Parameters:
    -----------
    data : dict
        Dictionary with data returned from fits files.

        The required keys are:

        ==========  ========================================================
        keys        Description
        ----------  --------------------------------------------------------
        flux        list of lists : Flux per pixel per order.
        wave        list of lists : Wavelength calibrated for BERV and RV
                    (at rest frame) per pixel per order [angstroms].
        blaze       list of lists : Blaze function.
        snr         list : SNR at each spectral order.
        obj         str : Object identification.
        date        str : Date of observation in the fits file format.
        ==========  ========================================================

    sel_lines : dict
        Dictionary containing the identification of the indices selected and
        the parameters of the spectral lines required for each index.

        The required keys are:

        ==========  ========================================================
        keys        Description
        ----------  --------------------------------------------------------
        ind_id      str : Index identification.
        ind_var     str : Variable assigned to a given line to be used in
                    the index equation. Ex: 'L1', 'L2', etc, for the core
                    lines, and 'R1', 'R2', etc, for reference lines.
        ln_id       str : Spectral line identification.
        ln_ctr      float : Wavelength of the line centre [angstroms].
        ln_win      float : Bandpass around the line centre to be used in
                    the flux integration [angstroms].
        bandtype    str : Type of bandpass used to integrate flux.
        ==========  ========================================================

    ln_plts : {str, False} (optional)
        Path for the saved line plots. If False, the plots are not saved
        (default).
    frac : bool (optional)
        Use fractional pixels if 'True' (default), use integral pixels if 'False'.

    Returns:
    --------
    sel_lines : dict
        Dictionary containing the identification of the indices selected and
        the parameters of the spectral lines required for each index.

        Included in the output are also the keys used as input (see above).

        The returned keys are:

        ==========  ========================================================
        keys        Description
        ----------  --------------------------------------------------------
        flux        list : Integrated flux for each line.
        error       list : Errors on integrated flux for each line.
        snr         list : SNR for each line.
        flg         list : Flag for each line, 'negFlux' if negative flux
                    found inside any bandpass.
        frac_neg    list : Fraction of pixels with negative values of flux
                    for each line.
        npixels     float : Number of pixels inside the bandpass.
        ==========  ========================================================
    """

    print()
    print("CALCULATING FLUX IN SELECTED LINES")
    print("----------------------------------")

    wave = data['wave']
    flux = data['flux']
    blaze = data['blaze']

    obj = data['obj']
    obs_date = data['obs_date']
    snr = data['snr']

    sel_lines['flux'] = []
    sel_lines['error'] = []
    sel_lines['snr'] = []
    sel_lines['flg'] = []
    sel_lines['frac_neg'] = []
    sel_lines['npixels'] = []

    ln_ctr = sel_lines['ln_ctr']
    ln_win = sel_lines['ln_win']
    ln_id = sel_lines['ln_id']
    ind_var = sel_lines['ind_var']
    bandtype = sel_lines['bandtype']

    if 'error_pixel' in list(data):
        # case where pixel errors are given in rdb file as "error_pixel"
        print("Using pixel errors from input rdb file.")
        err = data['error_pixel']
    else: err = None

    for k in range(len(ln_id)):
        print()
        print("Computing flux in line {}".format(ln_id[k]))
        print("-----------------------{}".format('-'*len(ln_id[k])))

        print("Using {} bandpass".format(bandtype[k]))

        win = ac_get_win.get_win(data, ln_id[k], ln_ctr[k], ln_win[k], bandtype[k], blaze=blaze, err=err, frac=frac, ln_plts=ln_plts)

        sel_lines['flux'].append(win['sum'])
        sel_lines['error'].append(win['sum_err'])
        sel_lines['snr'].append(win['snr'])
        sel_lines['flg'].append(win['flg'])
        sel_lines['frac_neg'].append(win['frac_neg'])
        sel_lines['npixels'].append(win['npixels'])

    return sel_lines


def calc_ind(sel_lines):
    """
    Calculates the indices identified in sel_lines as 'ind_id'.

    sel_lines : dict
        Dictionary containing the identification of the indices selected and
        the parameters of the spectral lines required for each index.

        The required keys are:

        ==========  ========================================================
        keys        Description
        ----------  --------------------------------------------------------
        ind_id      str : Index identification.
        ind_var     str : Variable assigned to a given line to be used in
                    the index equation. Ex: 'L1', 'L2', etc, for the core
                    lines, and 'R1', 'R2', etc, for reference lines.
        ln_id       str : Spectral line identification.
        flux        list : Integrated flux for each line.
        error       list : Errors on integrated flux for each line.
        snr         list : SNR for each line.
        flg         list : Flag for each line, 'negFlux' if negative flux
                    found inside any bandpass.
        frac_neg    list : Fraction of pixels with negative values of flux
                    for each line.
        npixels     float : Number of pixels inside the bandpass.
        ==========  ========================================================

    Returns:
    --------
    index : dict
        Dictionary containing the index values, errors and related
        information.

        The returned keys are:

        ==========  ========================================================
        keys        Description
        ----------  --------------------------------------------------------
        index       str : Index identification.
        value       float : Index value.
        error       float : Index error.
        flg         {str, None} : Index flag, 'negFlux' if negative flux
                    found inside any bandpass.
        mfrac_neg   float : Maximum fraction of pixels with negative values
                    of flux taking into account all lines used to compute
                    index.
        snr         float : Mean SNR of all lines used to compute index.
        npixels     float : Number of pixels inside the bandpass.
        ==========  ========================================================
    """

    print()
    print("CALCULATING INDICES")
    print("-------------------")

    # remove duplicates of ind_id and gives a list of selected indices
    sel_ind = list(set(sel_lines['ind_id']))
    sel_ind = np.asarray(sel_ind)

    index = {}
    index['index'] = []
    index['value'] = []
    index['error'] = []
    index['flg'] = []
    index['mfrac_neg'] = []
    index['snr'] = []
    npixels_list = []

    print("index\tvalue\terror\t\tsnr\tflag\tmfrac_neg")
    print("-----\t-----\t-----\t\t---\t----\t---------")

    ind_ids = np.asarray(sel_lines['ind_id'])
    rows = len(sel_lines['ln_id'])
    for i in range(len(sel_ind)): # each index

        var = [sel_lines['ind_var'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]]
        flux = [sel_lines['flux'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]]
        err = [sel_lines['error'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]]
        flg = [sel_lines['flg'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]]
        frac_neg = [sel_lines['frac_neg'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]]
        snr = [sel_lines['snr'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]]
        ln_c = [sel_lines['ln_c'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]] ##
        npixels = [sel_lines['npixels'][k] for k in range(rows) \
                                                if ind_ids[k] == sel_ind[i]]

        # Maximum fraction of flux with negative values of all lines
        mfrac_neg = max(frac_neg)

        if "negFlux" in flg: flg_ind = 'negFlux'
        else: flg_ind = None

        # Mean snr of lines:
        try: snr_ind = np.mean([float(snr[k]) for k in range(len(snr))])
        except: snr_ind = None

        for k in range(len(var)):
            if 'L' not in var[k] and 'R' not in var[k]:
                msg="*** ERROR: 'ind_var' variable (in config file config_lines.txt) must start with either an 'L' for core line or 'R' for reference line. Value given was '{}'".format(var[k])
                sys.exit(msg)

        # Add line variables for numerator or denominator:
        num = [ln_c[k]*flux[k] for k in range(len(var)) if 'L' in var[k]]
        num_err = [ln_c[k]*err[k] for k in range(len(var)) if 'L' in var[k]]
        denom = [ln_c[k]*flux[k] for k in range(len(var)) if 'R' in var[k]]
        denom_err = [ln_c[k]*err[k] for k in range(len(var)) if 'R' in var[k]]

        num = np.asarray(num)
        denom = np.asarray(denom)
        num_err = np.asarray(num_err)
        denom_err = np.asarray(denom_err)

        ind = sum(num) / sum(denom)

        # Error using propagation of errors for lines and ref lines
        ind_err = np.sqrt(sum(num_err**2) + ind**2 * sum(denom_err**2)) /sum(denom)

        if snr_ind is not None: snr_ind = round(snr_ind, 2)

        index['index'].append(sel_ind[i])
        index['value'].append(ind)
        index['error'].append(ind_err)
        index['flg'].append(flg_ind)
        index['mfrac_neg'].append(mfrac_neg)
        index['snr'].append(snr_ind)
        npixels_list.append(npixels)

        print("{}\t{:.4f}\t{:.6f}\t{}\t{}\t{:.4f}".format(index['index'][i], index['value'][i], index['error'][i], index['snr'][i], index['flg'][i], index['mfrac_neg'][i]))

    # npixels data
    for k in range(len(sel_lines['ln_id'])):
        index['{}_npixels'.format(sel_lines['ln_id'][k])] = sel_lines['npixels'][k]

    return index



#!/usr/bin/env python

# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import os, sys
import codecs

def read_conf(config_file, calc_index):
    """
    Reads data from config file and selects the lines needed for the
    selected indices.

    Parameters:
    -----------
    config_file : string
    	Name of the configuration file with path.
    calc_index : list of strings
        List of index ids to be calculated selected from the indices provided in the configuration file.

    Returns:
    --------
    sel_lines : dictionary
    	Dictionary containing the identification of the indices selected and
    	the parameters of the spectral lines required for each index.

    	Each key entry is a list of parameters where the list indices form the
    	rows related to the same spectral line identified with the key 'ln_id'
    	which is related to the spectral index identified by 'ind_id'.

    	The returned keys are:

    	==========  ========================================================
    	keys		Description
    	----------  --------------------------------------------------------
    	ind_id		str : Index identification.
    	ind_var		str : Variable assigned to a given line to be used in
    				the index equation. Ex: 'L1', 'L2', etc, for the core
    				lines, and 'R1', 'R2', etc, for reference lines.
    	ln_id		str : Spectral line identification.
        ln_c		float : Constant to be multilied to the flux of the line.
    	ln_ctr 		float : Wavelength of the line centre [angstroms].
    	ln_win 		float : Bandpass around the line centre to be used in
    				the flux integration [angstroms].
        bandtype    str : Type of bandpass used to integrate flux.
    	==========  ========================================================
    """

    print()
    print("LOADING DATA FROM CONFIG FILE")
    print("-----------------------------")

    try:
        f = codecs.open(config_file, 'r', encoding="utf-8")
    except FileNotFoundError as fnf_err:
        print("*** ERROR: Config file not found:")
        print(fnf_err)
        print("***", config_file)
        sys.exit()
    except TypeError as ty_err:
        print("*** ERROR: Config file is not 'str'")
        print("***", ty_err)
        sys.exit()

    # Ignores commented lines, reads header, then stops when dash is found
    for line in f:
        if line.startswith('#'): pass
        elif line.startswith('-'): break
        else: header = line

    # Convert the lines in the config table into list of strings
    columns = []
    for line in f:
    	if not line.strip(): pass # ignore empty new lines
    	else:
            line = line.strip() # removes whites spaces at the start and end of line
            column = line.replace("\t\t", "\t") # converts double tabs to single tabs
            column = line.split() # splits the line into a list of strings
            columns.append(column)

    f.close()

    # Associate each key (header) in table to a column
    lines = {}
    keys = header.split() # keys as provided in the table in config file
    for k in range(len(keys)):
    	lines[keys[k]] = []
    	for i in range(len(columns)):
    		lines[keys[k]].append(columns[i][k])

    # Converts numerical values in the list to floats and leaves the text strings
    for k in range(len(lines)):
        for i, x in enumerate(lines[keys[k]]):
            try: lines[keys[k]][i] = float(x)
            except ValueError: pass

    # Get indices ids from the function arguments
    sel_ind = calc_index

    # Check weather the selected indices have lines in config table
    for k in range(len(sel_ind)):
        if sel_ind[k] not in lines['ind_id']:
            sys.exit("*** ERROR: Index {} is not in the config file.".format(sel_ind[k]))

    #Iinitiate a dictionary with keys as the headers in table and make empty lists
    sel_lines = {}
    for i in range(len(keys)): sel_lines[keys[i]] = []

    # Select the rows that belong to the selected index
    rows = len(lines['ind_id'])
    for k in range(rows):
    	if lines['ind_id'][k] in sel_ind:
    		print(lines['ln_id'][k])
    		for i in range(len(keys)):
    			sel_lines[keys[i]].append(lines[keys[i]][k])
    	else: pass

    return sel_lines



#!/usr/bin/env python


# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import sys, os
import glob
import time
import datetime
import numpy as np
import astropy.io.fits as pyfits

import argparse
import pkg_resources
import appdirs

# Directory of this file
path = os.path.dirname(os.path.realpath(__file__))

# Global variables
#import ac_settings as ac_set

# Location of ACTIN files:
actin_files_dir = os.path.join(path, "actin_files")
sys.path.append(actin_files_dir)
# Global variables
import ac_settings as ac_set

import ac_config
import ac_read_data
import ac_get_win
import ac_calc_ind
import ac_save
import ac_plot_time as ac_plot
import ac_tools

from matplotlib import pylab as plt #####


# initiate global variables:
ac_set.init()
# Configuration file:
config_file = os.path.join(path, "config_lines.txt")
# Version file:
version_file = os.path.join(path, "VERSION")
# Print preamble:
version = ac_set.preamble(version_file)


def actin_file(file, calc_index=None, rv_in=None, config_file=config_file, save_output=False, ln_plts=False, obj_name=None, targ_list=None, del_out=False, frac=True):
    """
    Runs ACTIN for one fits file.
    Accepts files of types: 'S2D', 'S1D', 'e2ds', 's1d', 's1d_*_rv', 'ADP', and 'rdb'.
    Recognizes fits files from HARPS, HARPS-N and ESPRESSO instruments.
    """

    print()
    print("--------------------")
    print("EXECUTING ACTIN_FILE")
    print("--------------------")

    if type(file) is list: file = file[0]

    # Check if file is from known instrument
    tel, instr = ac_tools.get_instr(file)
    if instr == False: pass
    elif instr in ac_set.instr: pass
    else:
        msg="*** ERROR:\nUnrecognized instrument. ACTIN only accepts HARPS, HARPS-N or ESPRESSO. To read from a different instrument convert data to rdb file with the headers: 'obj', 'obs_date', 'bjd', 'wave', 'flux', 'error_pixel' (optional)."
        sys.exit(msg)

    # Checking if object in targ_list is the same as the object in fits file
    if targ_list:
        check = ac_tools.check_targ(file, targets=targ_list)
        if check is True: pass
        elif check is False: return

    # Read config file and retrieve lines information
    if calc_index:
        sel_lines = ac_config.read_conf(config_file, calc_index)

    # Read data from file
    data = ac_read_data.read_data(file, rv_in=rv_in, obj_name=obj_name)

    if not data: return

    # Check output file for duplicates
    if save_output is not False and data['file_type'] != "rdb":
        dupl = ac_save.check_duplicate(data['obj'], data['obs_date'], data['instr'], data['file_type'], save_output)
        if dupl is True: return

    if calc_index:
        # Check selected lines for spectral range and orders
        ac_calc_ind.check_lines(data['wave'], sel_lines)

        # Calculate flux in the required lines
        sel_lines = ac_calc_ind.calc_flux_lines(data, sel_lines, ln_plts=ln_plts, frac=frac)

        # Calculate chosen indices
        index = ac_calc_ind.calc_ind(sel_lines)
    if not calc_index:
        index = None
        sel_lines = None

    # Write output to rdb file in "out_dir"/"obj"
    if save_output is not False:
        rdb_file = ac_save.save_data(data, index, out_dir=save_output)
    else: rdb_file = None

    info = {}
    info['config_file'] = config_file
    info['file_type']   = data['file_type']
    info['version']     = version
    info['source_path'] = os.path.split(file)[0]
    info['tel']         = data['tel']
    info['instr']       = data['instr']
    info['obj']         = data['obj']

    options = {}
    options['frac'] = frac

    output = {}
    output['data']      = data
    output['index']     = index
    output['sel_lines'] = sel_lines
    output['info']      = info
    output['options']   = options
    output['rdb_file']  = rdb_file

    return output


def actin(files, calc_index=None, rv_in=None, config_file=None, save_output=False, ln_plts=False, obj_name=None, targ_list=None, del_out=False, frac=True, test=False, save_plots=False):
    """
    Runs 'actin_file' function for one or multiple fits files, for one or multiple stars.
    Accepts fits files from HARPS, HARPS-N, and ESPRESSO instruments.
    Accepts files of types: 'S1D', 'S2D', 'e2ds', 's1d', 's1d_*_rv', 'ADP', and 'rdb'.
    """

    print()
    print("----------------")
    print(" STARTING ACTIN ")
    print("----------------")

    start_time = time.time()

    # Get config file from installation or input
    if config_file is None:
        cfg_file = get_config()
    else:
        cfg_file = config_file
    print()
    print("Using spectral lines from configuration file:")
    print(cfg_file)

    # test values can be 'S1D', 'S2D', 'e2ds', 's1d', 'adp', or 'rdb'
    # this needs to have before anything that uses 'files'
    if test:
        calc_index, files = ac_tools.test_actin(test, path, calc_index)

    if not files:
        sys.exit()

    # Make lists to be iterated below
    if isinstance(files, str): files = [files]

    if rv_in is None:
        rv_in = [rv_in]*len(files)
    elif type(rv_in) is not list:
        rv_in = [float(rv_in)] ### added float to use as module
    else: pass

    # Check if files exist
    ac_tools.check_files(files)

    # Remove output file
    if del_out:
        ac_tools.remove_output(files, save_output, targ_list)

    # Option to make line plots directory the same as the data output dir
    if ln_plts == 'same':
        ln_plts = save_output

    total_files = len(files)

    # Organize files by path to star and file type
    files_list = ac_tools.files_by_star_and_ftype(files)

    n_files_t = 0
    # star directories
    for k in range(len(files_list)):
        # file types
        for i in range(len(files_list[k])):
            n_files = 0
            # files organized by dir and file type
            for j in range(len(files_list[k][i])):
                n_files += 1
                n_files_t += 1
                # Run actin file
                output = actin_file(files_list[k][i][j],
                                    calc_index,
                                    rv_in=rv_in[j],
                                    config_file=cfg_file,
                                    save_output=save_output,
                                    ln_plts=ln_plts,
                                    obj_name=obj_name,
                                    targ_list=targ_list,
                                    del_out=del_out,
                                    frac=frac)

            # POST-PROCESSING:
            if output:
                # Dictionaries for each file type
                sel_lines = output['sel_lines']
                info = output['info']
                options = output['options']
                rdb_file = output['rdb_file']

                # Save log and line info files
                ac_save.save_log(info, options, n_files, out_dir=save_output)
                ac_save.save_line_info(info, sel_lines, out_dir=save_output)

                if save_plots:
                    # Save time-series plots
                    ac_plot.plt_time(info, out_dir=save_output, rmv_flgs=False, save_plt=True)
                    ac_plot.plt_time_mlty(info, out_dir=save_output, rmv_flgs=False, save_plt=True, hdrs=calc_index)
            else: pass

    if n_files_t != total_files:
        print()
        print("*** ERROR: Number of ACTIN calls different than number of files.")
        print("n_files_t:", n_files_t)
        print("total_files:", total_files)

    elapsed_time = (time.time() - start_time)/60

    # Summary:
    print("\n---------------------------------")
    print("Fractional pixels:\t{}".format(frac))
    print("Files analysed:\t\t{}".format(total_files))
    print("Save output:\t\t{}".format(save_output))
    print("Elapsed time:\t\t{:.4f} min".format(elapsed_time))

    return


def get_config():
    """
    Check for existence of ACTIN folder and config file and creates them if not present. Returns the path to the config file.
    """
    cfg_dir = appdirs.user_config_dir('ACTIN')
    if not os.path.exists(cfg_dir):
        os.makedirs(cfg_dir)
    cfg_file = os.path.join(cfg_dir, 'config_lines.txt')
    if not os.path.isfile(cfg_file):
        create_user_config(cfg_file)

    return cfg_file


def create_user_config(cfg_file):
    """
    Create the user's config file
    """
    from shutil import copyfile ###
    src = pkg_resources.resource_stream(__name__, 'config_lines.txt')
    copyfile(src.name, cfg_file)



def main():
    """
    Main function, call actin function with arguments from terminal.
    """

    # initiate the parser
    parser = argparse.ArgumentParser()

    # add short and long argument
    parser.add_argument('--files', '-f', help='Read file(s)', nargs='+')

    parser.add_argument('--calc_index', '-i', help="Index id to calculate as designated by 'ind_id' in config_index.txt.", nargs='+', default=None)

    parser.add_argument('--rv_in', '-rv', help="RV value to calibrate wavelength. If False (default) try to read RV from CCF file.", nargs='+', default=None, type=float)

    parser.add_argument('--config_file', '-cf', help='Path to config_file, or False (default) read config file from standard directory.', default=None)

    parser.add_argument('--save_output', '-s', help='Path to output directory of data table, or False (default).', default=False)

    parser.add_argument('--ln_plts', '-lp', help="Path to directory to save line plots. If 'same' saves line plots to same directory of data output. If 'show' only shows the plots. If 'False' (default) does not save or show line plots", default=False)

    parser.add_argument('--obj_name', '-obj', help='Give target a name that overides the one from the fits files.', default=None)

    parser.add_argument('--targ_list', '-tl', help='Give a list of stars to select from fits files.', nargs='+', default=None)

    parser.add_argument('--del_out', '-del', help='Delete output data file if True.', default=False, type=lambda x: (str(x).lower() == 'true'))

    parser.add_argument('--test', '-t', help='Tests actin using the provided fits files in the "test_files" directory. Options are "e2ds", "s1d", and "adp"', default=False)

    parser.add_argument('--frac', '-frc', help='Turns fractional pixel on (True, default) or off (False).', default=True, type=lambda x: (str(x).lower() == 'true'))

    parser.add_argument('--save_plots', '-sp', help="If True saves time-series and multi-plots to same directory as 'save_output'.", default=False, type=lambda x: (str(x).lower() == 'true'))


    # read arguments from the command lines
    args = parser.parse_args()

    actin(files       = args.files,
          calc_index  = args.calc_index,
          rv_in       = args.rv_in,
          config_file = args.config_file,
          save_output = args.save_output,
          ln_plts     = args.ln_plts,
          obj_name    = args.obj_name,
          targ_list   = args.targ_list,
          del_out     = args.del_out,
          test        = args.test,
          frac        = args.frac,
          save_plots  = args.save_plots)

if __name__ == "__main__":
    main()



#!/usr/bin/env python

# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import os, sys
import string
from pylab import *
import astropy.io.fits as pyfits
import numpy as np
import glob

from matplotlib import pylab as plt

import ac_settings as ac_set


def compute_flux(wave, flux, blaze, noise, ln_ctr, ln_win, bandtype, frac=True, test_plot=False):
    """
    Calculates the flux inside a bandpass.
    """

    print("Executing compute_flux")

    if blaze is None: blaze = np.ones(len(flux))

    # make all important values the size of px_size
    px_size = np.diff(wave)
    #px_size = [float(px_size[k]) for k in range(len(px_size))] ####
    #px_size = np.asarray(px_size) ####
    wave  = wave[1:]
    flux  = flux[1:]
    blaze = blaze[1:]

    # make all values float otherwise python will not detect very small differences between wave and wmin/max
    #wave = [float(wave[k]) for k in range(len(wave))] ###
    #wave = np.asarray(wave) ####

    # BANDPASS TYPES
    if bandtype == 'tri':
        #wmin = float(ln_ctr - ln_win)
        #wmax = float(ln_ctr + ln_win)
        wmin = ln_ctr - ln_win #########
        wmax = ln_ctr + ln_win #########
        cond = (wave > wmin) & (wave < wmax)
        bandfunc = -np.abs(wave-ln_ctr)/ln_win + 1.
        bandfunc = np.where(bandfunc > 0, bandfunc, bandfunc*0.0)
    if bandtype == 'sq':
        #wmin = float(ln_ctr - ln_win/2.)
        #wmax = float(ln_ctr + ln_win/2.)
        wmin = ln_ctr - ln_win/2. #########
        wmax = ln_ctr + ln_win/2. #########
        cond = (wave > wmin) & (wave < wmax)
        bandfunc = np.ones(len(wave))
        bandfunc = np.where(cond, 1., 0.)

    # HARPS METHOD
    if frac == False:
        flux_win     = flux[cond]
        wave_win     = wave[cond]
        blaze_win    = blaze[cond]
        flux_win_deb = flux[cond]/blaze[cond]
        px_size_win  = px_size[cond]
        npixels      = len(wave[cond])
        response     = bandfunc[cond]
    # ACTIN METHOD
    if frac == True:
        dwave_l = wave[cond][0]-wmin
        dwave_r = wmax-wave[cond][-1]
        dwave = dwave_l + dwave_r

        cond_l = (wave < wave[cond][0])
        cond_r = (wave > wave[cond][-1])

        frac_l = dwave_l/px_size[cond_l][-1]
        frac_r = dwave_r/px_size[cond_r][0]

        npixels = len(wave[cond]) + frac_l + frac_r

        # just for visualization purposes
        wave_win  = np.r_[wave[cond][0]-dwave_l, wave[cond], wave[cond][-1]+dwave_r]
        flux_win  = np.r_[flux[cond_l][-1], flux[cond], flux[cond_r][0]]
        blaze_win = np.r_[blaze[cond_l][-1], blaze[cond], blaze[cond_r][0]]

        px_size_win = np.r_[px_size[cond_l][-1], px_size[cond], px_size[cond_r][0]]

        response = bandfunc[cond]
        if bandtype == 'sq':
            response = np.r_[1*frac_l, response, 1*frac_r]
        if bandtype == 'tri':
            response = np.r_[0.0, response, 0.0]

        flux_win_deb = flux_win/blaze_win

        # tests
        bandwidth = dwave_l + wave[cond][-1]-wave[cond][0] + dwave_r
        if bandwidth == (wmax - wmin): pass
        else: print("*** bandwidth test 1 ERROR")

        bandwidth = frac_r*px_size_win[-1] + sum(px_size[cond][1:]) + px_size_win[0]*frac_l
        if bandwidth == (wmax - wmin): pass
        else: print("*** bandwidth test 2 ERROR")

    # Flux sum and variance for line:

    if not noise: noise = 0.0
    f_sum     = sum(flux_win_deb*response/sum(response*px_size_win))
    f_sum_var = sum((flux_win+noise**2)*response**2/blaze_win**2)/sum(response*px_size_win)**2

    print("Pixels in bandpass = %.4f" % npixels)

    # Flag negative flux inside bandpass
    flg, frac_neg = flag_negflux(flux_win)

    # Test plots
    if test_plot == True:
        w_out_l = wave[wave < wave[cond][0]][-1]
        w_in_l  = wave[cond][0]
        w_out_r = wave[wave > wave[cond][-1]][0]
        w_in_r  = wave[cond][-1]

        plt.plot(wave, flux, 'kx')
        plt.plot(wave_win, flux_win*response, 'b.')
        plt.plot(wave_win, response*max(flux_win),c='orange',ls='-',linewidth=2)

        plt.axhline(0.0, c='k', ls=':')

        plt.axvline(wmin, color='k', ls='-')
        plt.axvline(w_in_l, color='r', ls=':')
        plt.axvline(w_out_l, color='g', ls='--')

        plt.axvline(wmax, color='k', ls='-')
        plt.axvline(w_in_r, color='r', ls=':')
        plt.axvline(w_out_r, color='g', ls='--')
        plt.show()

    return f_sum, f_sum_var, bandfunc, npixels, flg, frac_neg


def flag_negflux(flux):
    """
    Tests if flux has negative values and returns flag 'flg' as 'negFlux' if detected, None otherwise, and the fraction of pixels with negative values of flux, 'frac_neg'.
    """
    negflux_array = np.where(flux < 0.0, flux, 0.0)

    negflux_only = [negflux_array[x] for x in range(len(negflux_array)) if negflux_array[x] < 0.0]

    # fraction of pixels with negative flux
    frac_neg = len(negflux_only)/len(flux)

    flag_array = np.where(flux < 0.0, 'negFlux', None)

    if 'negFlux' in flag_array:
        flg = 'negFlux'
        print("*** WARNING: Negative flux detected")
        print("Fraction of pixels with negative values = {:.5f}".format(frac_neg))
    else: flg = None

    return flg, frac_neg


def remove_output(files, save_output, targ_list=None):
    """
    Removes output directories for given fits files 'files', 'save_output' directory and list of targets 'targ_list' (if available).
    """

    print()
    print("Executing actin_functions.remove_output:")
    print("Searching output files to delete...")

    fn_rdb = ac_set.fnames['data']

    rdb_search = os.path.join(save_output, "*", "*{}".format(fn_rdb))
    rdb_files = glob.glob(rdb_search)

    if not rdb_files:
        print("There are no files to remove.")
        return

    obj_list = []
    for file in files:
        obj_list.append(get_target(file))
    objs = list(set(obj_list))

    for obj in objs:
        for rdb_file in rdb_files:
            if obj in rdb_file:
                if targ_list is not None:
                    if obj in targ_list:
                        os.remove(rdb_file)
                        print("Output file removed:")
                        print(rdb_file)
                else:
                    os.remove(rdb_file)
                    print("Output file removed:")
                    print(rdb_file)

    return


def files_by_star_and_ftype(files):
    """
    Organize files by file type.
    """
    # different file types in files
    ft = []
    for k in range(len(files)):
        ft.append(get_file_type(files[k]))
    ft = list(set(ft))

    # different stars in files
    sp = []
    for k in range(len(files)):
        sp.append(os.path.split(files[k])[0])
    sp = list(set(sp))

    files_list = []
    for k in range(len(sp)):
        lists_sp = []
        for i in range(len(ft)):
            lists_ft = []
            for j in range(len(files)):
                if sp[k] in files[j] and ft[i] in files[j]:
                    lists_ft.append(files[j])
            if lists_ft:
                lists_sp.append(lists_ft)
        files_list.append(lists_sp)

    return files_list


def override_obj(obj, obj_name):
    """
    Override object name with name given in obj_name option.
    """
    if type(obj_name) is list and len(obj_name) == 1:
        return obj_name[0]
    elif type(obj_name) is list and len(obj_name) > 1:
        sys.exit("*** ERROR: obj_name requires only one name, more than one given.")
    else: return obj_name


def check_files(files):
    for k in range(len(files)):
        if not os.path.isfile(files[k]):
            sys.exit("*** ERROR: File {} not found.".format(files[k]))


def get_file_type(file):
    """
    Checks file name for known file types and returns it.
    """
    for k in range(len(ac_set.ftypes['all'])):
        if ac_set.ftypes['all'][k] in file:
            return ac_set.ftypes['all'][k]


def get_instr(fits_file):
    if ".rdb" not in fits_file:
        hdu = pyfits.open(fits_file)

        tel = hdu[0].header['TELESCOP']
        instr = hdu[0].header['INSTRUME']

        hdu.close()

        return tel, instr
    if ".rdb" in fits_file: return False, False


def get_target(fits_file):
    """
    Returns the object targeted in the fits file 'fits_file'.
    """
    hdu = pyfits.open(fits_file)

    try:
        obj = hdu[0].header['OBJECT']
        hdu.close()
    except:
        try:
            obj = hdu[0].header['ESO OBS TARG NAME']
            hdu.close()
        except:
            try:
                obj = hdu[0].header['TNG OBS TARG NAME']
                hdu.close()
            except:
                print("*** ERROR: Cannot identify object.")
                return

    return obj


def check_targ(fits_file, targets):
    """
    Checks if a fits file belongs to a target in a list of targets.
    """
    print("Executing: check_targ")
    print("Targets = {}".format(targets))

    obj = get_target(fits_file)

    print("Object = {}".format(obj))
    if obj in targets: return True
    else:
        print("*** INFO: {} is not in the target list.".format(obj))
        print("*** INFO: file {}".format(fits_file))
        return False


def test_actin(test, path, calc_index):
    if not calc_index:
        calc_index = ("I_CaII", "I_Ha", "I_NaI")
    if test == "S1D":
        files = os.path.join(path, "test_files", "HD41248_1_1_S1D_A.fits")
    elif test == "S2D":
        files = os.path.join(path, "test_files", "HD41248_1_1_S2D_A.fits")
    elif test == "e2ds":
        files = os.path.join(path, "test_files", "HARPS.2003-12-13T06:19:48.371_e2ds_A.fits")
    elif test == "s1d":
        files = os.path.join(path, "test_files", "HARPS.2010-09-18T23:42:36.178_s1d_A.fits")
    elif test == "adp":
        files = os.path.join(path, "test_files", "ADP.2014-09-16T11:04:45.123.fits")
    elif test == 'rdb':
        files = os.path.join(path, "test_files", "2010-09-18T23:42:36.178_spec.rdb")
    else:
        print("*** ERROR:")
        print("*** Test can only be 'S1D', 'S2D', 'e2ds', 's1d', 'adp', or 'rdb'")
        return
    return calc_index, files


def read_rdb(filename):
    """
    Reads an .rdb file and returns a dictionary with the headers as keys and data as lists ('output'), and also a list of headers ('keys').

    use: table = pyrdb.read_rdb(file)[0] for data
    use: table = pyrdb.read_rdb(file)[1] to get the keys by order
    """
    f = open(filename, 'r')
    data = f.readlines()
    f.close()

    key = str.split(data[0][:-1],'\t')
    output = {}
    for i in range(len(key)): output[key[i]] = []

    for line in data[2:]:
        qq = str.split(line[:-1],'\t')
        for i in range(len(key)):
            try: value = float(qq[i])
            except ValueError: value = qq[i]
            output[key[i]].append(value)

    return output, key


def save_rdb(dic, keys,file):
    """
    From a disctionary 'dic' saves the columns related to the specified 'keys' into an .rdb file named 'file'.
    """
    out = open(file,'w')
    n_keys = len(keys)

    for k in range(n_keys):
        if k != n_keys-1: out.write('{}\t'.format(keys[k]))
        else: out.write('{}'.format(keys[k]))
    out.write('\n')
    for k in range(n_keys):
        if k != n_keys-1: out.write('{}\t'.format('-'*len(keys[k])))
        else: out.write('{}'.format('-'*len(keys[k])))
    out.write('\n')

    for i in range(len(dic[keys[0]])):
        for k in range(n_keys):
            if k != n_keys-1: out.write('{}\t'.format(dic[keys[k]][i]))
            else: out.write('{}'.format(dic[keys[k]][i]))
        out.write('\n')
    out.close()


def add_rdb(dic,keys, file_name):
    """
    Adds data to an existing .rdb file. The 'keys' must match the headers already present in the file.
    """
    out = open(file_name,'a')
    n_keys = len(keys)
    for i in range(len(dic[keys[0]])):
        for k in range(n_keys):
            if k != n_keys-1: out.write('{}\t'.format(dic[keys[k]][i]))
            else: out.write('{}\t'.format(dic[keys[k]][i]))
        out.write('\n')
    out.close()


def plot_params(width=6, height=3.5):
        """
        Parameters for plots.
        """
        rcdefaults()
        rc('axes', linewidth=1)
        rc('lines', markeredgewidth=0.5)

        rc('errorbar', capsize=2)

        rc('ytick', direction='in')
        rc('xtick', direction='in')

        rc('ytick', right='True')
        rc('xtick', top='True')

        rc('xtick.minor', visible='True')


        rc('xtick.major', size=7)
        rc('ytick.major', size=7)
        rc('xtick.minor', size=3)
        rc('ytick.minor', size=3)
        rc("font", family="sans-serif")
        rc("font", size=10)

        rc("figure.subplot", left=(0.15))
        rc("figure.subplot", right=(0.93))
        rc("figure.subplot", bottom=(0.12))
        rc("figure.subplot", top=(0.95))

        return width, height



#!/usr/bin/env python


# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import sys, os
import numpy as np
import glob

import astropy.io.fits as pyfits
import matplotlib.pylab as plt

# ACTIN modules:
import ac_tools
import ac_settings as ac_set


def check_for_calib_files(e2ds_header, file_type, folder, dif_time_max=1.0, plot_spec=False):
    """
    Check for calibration files (wave or blaze) in the working directory
    in case of predetermined files are not present and choses the ones with
    less than 1 day difference from the original e2ds file.

    Parameters:
    -----------
    e2ds_header : fits header
        Fits header of e2ds fits file.
    file_type : str
        File type to be searched, either 'wave' or 'blaze'.
    folder : str
        Path to the fits files.
    dif_time_max : float (optional)
        Maximum difference in time between e2ds and
        the chosen calib file [days].

    Returns:
    --------
    calib_pfile : {str, None}
        Selected calibration file with path included, None if
        file not found.
    """

    print("Executing: check_for_calib_files")

    mjd_e2ds = e2ds_header['MJD-OBS']

    filename = os.path.join(folder, '*{}_A.fits'.format(file_type))
    calib_pfiles = glob.glob(filename)
    if len(calib_pfiles) > 0:

        time_dif = []

        for k in range(len(calib_pfiles)):
            w = pyfits.open(calib_pfiles[k])
            mjd = w[0].header['MJD-OBS']
            w.close()
            time_dif.append(abs(mjd_e2ds-mjd))

        if min(time_dif) < dif_time_max:
            index = time_dif.index(min(time_dif))
            calib_pfile = calib_pfiles[index]
            calib_file = os.path.split(calib_pfile)[-1]

            print("New {} file used:\t{}".format(file_type, calib_file))
            print("{} file time difference to e2ds = {:.2f} days".format(file_type, min(time_dif)))
            return calib_pfile
        else:
            print("*** WARNING: Closest {} file was produced longer than {} day(s)".format(file_type, dif_time_max))
            return None
    else:
        print("*** WARNING: No more {} files in folder".format(file_type))
        return None


def calc_wave(e2ds_pfile, obs):
    """
    Compute wavelength from e2ds headers.

    Parameters:
    -----------
    e2ds_pfile : str
        e2ds file name with path to its location.
    obs : str
        Code related to instrument to be used in fits headers.

    Returns:
    --------
    wave : list of lists
        2d wavelength list where len(wave) is number of orders [angstroms].
    """

    print("Executing: calc_wave")

    e2ds = pyfits.open(e2ds_pfile)

    deg = e2ds[0].header['HIERARCH {} DRS CAL TH DEG LL'.format(obs)]

    ll_coeff = np.zeros((len(e2ds[0].data), deg + 1))

    # Read coefficients
    for i in range(len(e2ds[0].data)):
        for j in range(deg + 1):
            ll_coeff[i, j] = e2ds[0].header['HIERARCH {} DRS CAL TH COEFF '
                                    'LL{}'.format(obs, (j + (deg + 1)*i))]

    # Evaluate polynomials
    x = np.arange(e2ds[0].data.shape[1])  # Pixel array
    wave = np.zeros(e2ds[0].data.shape)  # Wavelength 2D array
    for i in range(len(wave)):
        wave[i] = np.poly1d(ll_coeff[i][::-1])(x)

    e2ds.close()

    return wave


def read_data_rdb(file):
    """
    Read spectrum from rdb file with the following headers:
    'obj','obs_date','bjd','wave','flux','error_pixel' (optional)
    """
    print("Reading file:\t{}".format(file))
    try: data, hdr = ac_tools.read_rdb(file)
    except:
        print("*** ERROR: Cannot read {}".format(file))
        sys.exit()

    print("Object:\t\t{}".format(data['obj'][0]))
    data['wave'] = np.asarray(data['wave'])
    data['flux'] = np.asarray(data['flux'])
    data['tel'] = "unknown"
    data['instr'] = "unknown"
    data['obj'] = data['obj'][0]
    data['obs_date'] = data['obs_date'][0]
    data['bjd'] = data['bjd'][0]
    data['blaze'] = np.ones(len(data['flux']))
    data['snr'] = None
    data['median_snr'] = None
    data['noise'] = 0.0

    return data


def read_data(pfile, rv_in=None, obj_name=None, force_calc_wave=False, plot_spec=False):
    """
    Reads data from 'S2D', 'S1D', 'e2ds', 's1d', 's1d_*_rv', 'ADP', and 'rdb' files.
    - force_calc_wave is for testing purposes only.
    - plot_spec is for testing purposes only.
    """

    print()
    print("READING DATA FROM FILE:")
    print("-----------------------")

    flg = None

    # Get file type
    file_type = ac_tools.get_file_type(pfile)

    # Read rdb file and return data
    if file_type == 'rdb':
        data = read_data_rdb(pfile)
        data['file_type'] = file_type
        return data

    folder, file = os.path.split(pfile)
    file_info = file.split('_')[0]

    print("Working folder:\t{}{}".format(folder, os.path.sep))
    print("Reading file:\t{}".format(file))

    hdu = pyfits.open(pfile)
    hdr = hdu[0].header

    # Reading headers
    tel = hdr['TELESCOP']
    instr = hdr['INSTRUME']
    date_obs = hdr['DATE-OBS']

    print("Telescope:\t{}".format(tel))
    print("Instrument:\t{}".format(instr))

    if instr == 'HARPS':
        obs = 'ESO'
        ords = 72
    elif instr == 'HARPN':
        obs = 'TNG'
        ords = 69
    elif instr == 'ESPRESSO':
        obs = 'ESO'
        ords = 170
    else:
        sys.exit("*** ERROR: Instrument not recognized. ACTIN only accepts HARPS, HARPS-N, and ESPRESSO fits files. If using another instrument make an '.rdb' table with the headers 'obj', 'obs_date', 'bjd', 'wave', 'flux', 'error_pixel' (optional) and run ACTIN on that file.")

    # get target
    obj = ac_tools.get_target(pfile)

    print("Object:\t\t{}".format(obj))

    if obj in ('WAVE,WAVE,THAR1','WAVE,WAVE,THAR2'):
        print('*** WARNING: File is ThAr flux.')
        print("*** ACTION: Ignoring Measurement.")
        return

    # Override object name with name given in obj_name option
    if obj_name:
        obj = ac_tools.override_obj(obj, obj_name)
        print("Object name changed to:", obj)

    if instr in ('ESPRESSO'):
        if file_type == "S2D":
            flux = hdu[1].data
            flux_err = hdu[2].data
            wave_raw = hdu[5].data # wavelength air
        if file_type == "S1D":
            data = hdu[1].data
            flux = data['flux']
            flux_err = data['error']
            wave_raw = data['wavelength_air']
        bjd = hdr['HIERARCH {} QC BJD'.format(obs)]
        snr = [hdr['HIERARCH {} QC ORDER{} SNR'.format(obs,k+1)] for k in range(ords)]
        snr = np.asarray(snr)
        median_snr = np.median(snr)
        try: bv = hdu['HIERARCH {} OCS OBJ BV'.format(obs)]
        except: bv = None
        try: airmass_end = hdr['HIERARCH {} TEL3 AIRM END'.format(obs)]
        except: airmass_end = None

    if instr in ('HARPS', 'HARPN'):
        bjd = hdr['HIERARCH {} DRS BJD'.format(obs)]
        berv = hdr['HIERARCH {} DRS BERV'.format(obs)]
        try: bv = hdr['HIERARCH {} OCS OBJ BV'.format(obs)]
        except: bv = None
        try: airmass_end = hdu['HIERARCH {} TEL AIRM END'.format(obs)]
        except: airmass_end = None
        if file_type in ("s1d", "e2ds"):
            flux = hdu[0].data
            flux_err = None
            hdu.close()
            snr = [hdr['HIERARCH {} DRS SPE EXT SN{}'.format(obs, k)] for k in range(ords)]
            snr = np.asarray(snr)
            median_snr = np.median(snr)
        if file_type == 'ADP':
            flux = hdu[1].data[0][1]
            flux_err = None
            wave_raw = hdu[1].data[0][0]
            hdu.close()
            median_snr = hdr['SNR']
            blaze = np.ones(len(flux))
            snr = None

    if file_type == 's1d':
        # dwave_raw is 0.01 ang
        wave_raw = hdr['CRVAL1'] + hdr['CDELT1']*np.arange(hdr['NAXIS1'])
        blaze = np.ones(len(flux))

    if file_type in ('e2ds', 'ADP'):
        if file_type != 'ADP':
            # Reading data from wave file
            wave_file = hdr['HIERARCH {} DRS CAL TH FILE'.format(obs)]
            print("Wave file:\t{}".format(wave_file))
            wave_pfile = os.path.join(folder, wave_file)
            if force_calc_wave == False:
                try:
                    wave_hdu = pyfits.open(wave_pfile)
                except:
                    print("*** INFO: Could not open", wave_pfile)
                    wave_pfile = check_for_calib_files(hdr, 'wave', folder)
                    try:
                        wave_hdu = pyfits.open(wave_pfile)
                    except:
                        print("*** INFO: Could not open:")
                        print("***", wave_pfile)
                        print("*** ACTION: Computing wavelength from", file_type)
                        wave_raw = calc_wave(pfile, obs)
                        wave_raw = np.asarray(wave_raw)
                    else:
                        wave_raw = wave_hdu[0].data
                        wave_hdu.close()
                else:
                    wave_raw = wave_hdu[0].data
                    wave_hdu.close()
            if force_calc_wave == True:
                print("*** ACTION: Computing wavelength from", file_type)
                wave_raw = calc_wave(pfile, obs)
                wave_raw = np.asarray(wave_raw)

        # Reading data from blaze file
        blaze_file = hdr['HIERARCH {} DRS BLAZE FILE'.format(obs)]
        print("Blaze file:\t{}".format(blaze_file))
        blaze_pfile = os.path.join(folder, blaze_file)
        try:
            blaze_hdu = pyfits.open(blaze_pfile)
        except:
            print("*** WARNING: The blaze file associated with this e2ds is not present.")
            print("*** Looking for other blaze files in the folder...")
            blaze_pfile = check_for_calib_files(hdr,'blaze',folder)
            try:
                blaze_hdu = pyfits.open(blaze_pfile)
            except:
                print("*** WARNING: Flux not deblazed. This can introduce artificial variations in the indices values.")
                if file_type == 'e2ds':
                    blaze = np.ones([len(flux),len(flux[0])])
                if file_type == 'ADP':
                    blaze = np.ones(len(flux))
                flg = 'noDeblazed'
            else:
                blaze = blaze_hdu[0].data
                blaze_hdu.close()
        else:
            blaze = blaze_hdu[0].data
            blaze_hdu.close()

    if instr in ('ESPRESSO'):
        rv = hdr['HIERARCH {} QC CCF RV'.format(obs)] # [km/s]
        rv_err = hdr['HIERARCH {} QC CCF RV ERROR'.format(obs)] # [km/s]
        fwhm = hdr['HIERARCH {} QC CCF FWHM'.format(obs)] # [km/s]
        fwhm_err = hdr['HIERARCH {} QC CCF FWHM ERROR'.format(obs)] # [km/s]
        cont = hdr['HIERARCH {} QC CCF CONTRAST'.format(obs)] # [%]
        cont_err = hdr['HIERARCH {} QC CCF CONTRAST ERROR'.format(obs)] # [%]
        try: bv = hdr['HIERARCH {} OCS OBJ BV'.format(obs)] # B-V
        except: bv = None
        berv = hdr['HIERARCH {} QC BERV'.format(obs)] # [km/s] barycentric correction

        rv = rv * 1000 # convert to m/s
        rv_err = rv_err * 1000
        berv = berv * 1000 # convert to m/s
        fwhm = fwhm * 1000 # convert to m/s
        fwhm_err = fwhm_err * 1000
        bis = None
        bis_err = None
        ccf_noise = 0.0

        if file_type == 'S2D': blaze = np.ones([len(flux),len(flux[0])])
        if file_type == 'S1D': blaze = np.ones([len(flux)])

    if instr in ('HARPS', 'HARPN'):
        # Reading data from CCF file

        if file_type == 'ADP':
            ccf_search = "{}.{}*ccf_*_A.fits".format(instr, date_obs[:-2])
        else:
            ccf_search = "{}*_ccf_*_A.fits".format(file_info[:-2])
        ccf_search = os.path.join(folder, ccf_search)
        try:
            ccf_pfile = glob.glob(ccf_search)[0]
            ccf_hdu = pyfits.open(ccf_pfile)
        except:
            print("*** WARNING: Could not find or open:")
            print("***", ccf_search)
            print("*** WARNING: No CCF data available.")
            rv = None; rv_err = None; fwhm = None; fwhm_err = None; cont = None; cont_err = None; ccf_noise = None; berv = None
        else:
            ccf_file = os.path.split(ccf_pfile)[-1]
            print("CCF file:\t{}".format(ccf_file))
            ccf_hdr = ccf_hdu[0].header
            ccf_hdu.close()

            rv = ccf_hdr['HIERARCH {} DRS CCF RVC'.format(obs)] # [km/s]
            rv_err = ccf_hdr['HIERARCH {} DRS DVRMS'.format(obs)] # [m/s]
            ccf_noise = ccf_hdr['HIERARCH {} DRS CCF NOISE'.format(obs)] # [km/s]
            fwhm = ccf_hdr['HIERARCH {} DRS CCF FWHM'.format(obs)] # [km/s]
            fwhm_err = None
            cont = ccf_hdr['HIERARCH {} DRS CCF CONTRAST'.format(obs)] # [%]
            cont_err = None

            rv = rv * 1000 # convert to m/s
            berv = berv * 1000 # convert to m/s
            fwhm = fwhm * 1000 # convert to m/s
            ccf_noise = ccf_noise * 1000 # convert to m/s

        # Reading data from BIS file
        if file_type == 'ADP':
            bis_search = "{}.{}*bis_*_A.fits".format(instr, date_obs[:-2])
        else:
            bis_search = "{}*_bis_*_A.fits".format(file_info[:-2])
        bis_search = os.path.join(folder, bis_search)
        try:
            bis_pfile = glob.glob(bis_search)[0]
            bis_hdu = pyfits.open(bis_pfile)
        except:
            print("*** WARNING: Could not find or open:")
            print("***", bis_search)
            print("*** WARNING: No BIS data available.")
            bis = None
            bis_err = None
        else:
            bis_file = os.path.split(bis_pfile)[-1]
            print("BIS file:\t{}".format(bis_file))
            bis_hdr = bis_hdu[0].header
            bis_hdu.close()
            bis = bis_hdr['HIERARCH {} DRS BIS SPAN'.format(obs)] # [km/s]
            bis = bis * 1000 # convert to m/s
            bis_err = None

    # Wavelength callibration:
    c = 299792458.0 # [m/s]

    # receiving rv from input
    if rv_in is not None: rv = rv_in * 1000 # convert to m/s

    if instr in ('ESPRESSO'):
        wave = wave_raw - rv * wave_raw / c

    if instr in ('HARPS', 'HARPN'):
        if file_type in ac_set.ftypes['1d']:
            # To know if reading s1a_A_rv files:
            rest_frame = file.split('_')[-1].split('.')[0]

            # If using s1d need rv to calibrate wavelength
            if not rv and rest_frame != 'rv':
                print("*** ERROR: No rv data available to calibrate wavelength.")
                return

            # If using s1d_*_rv the wavelength is already at rest frame
            if rest_frame == 'rv': wave = wave_raw

            # If using s1d use rv to calibrate wavelength
            if rv and rest_frame != 'rv':
                dwave = rv * wave_raw / c
                wave = wave_raw - dwave

        if file_type in ac_set.ftypes['2d']:
            if rv:
                dwave = (rv - berv) * wave_raw / c
                wave = wave_raw - dwave
            else:
                print("*** ERROR: No rv data available to calibrate wavelength.")
                return

    # Test plot
    plot_spec = False

    if plot_spec == True:
        ord = 6
        dif_wave = np.diff(wave_raw)
        wave = wave_raw[1:]
        if type(wave[0]) in (list, np.ndarray):
            wave = wave[ord]
            plt.xlabel("Wave_raw ord {} [Ang]".format(ord))
            plt.ylabel("diff_wave_raw ord {} [Ang]".format(ord))
        else:
            plt.xlabel("Wave_raw [Ang]")
            plt.ylabel("diff_wave_raw")

        dif_wave = np.diff(wave)
        wave = wave[1:]

        plt.plot(wave, dif_wave, 'k.')

        #plt.axvline(7877.08, c='b',ls='-')
        plt.show()


    data = {}
    data['flux'] = flux
    data['flux_err'] = flux_err
    data['wave'] = wave
    data['blaze'] = blaze
    data['obs_date'] = date_obs
    data['obj'] = obj
    data['bjd'] = bjd
    data['median_snr'] = median_snr
    data['snr'] = snr # 2d
    data['rv'] = rv # m/s
    data['rv_err'] = rv_err # m/s
    data['fwhm'] = fwhm # m/s
    data['fwhm_err'] = fwhm_err
    data['cont'] = cont
    data['cont_err'] = cont_err
    data['bis'] = bis
    data['bis_err'] = bis_err
    data['noise'] = ccf_noise # m/s
    data['airmass_end'] = airmass_end
    data['bv'] = bv
    data['tel'] = tel
    data['instr'] = instr
    data['file_type'] = file_type
    data['data_flg'] = flg

    return data



#!/usr/bin/env python

# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import sys, os
import numpy as np
from matplotlib import pylab as plt

import ac_tools
import ac_settings as ac_set


def sel_order(wave_2d, ln_ctr, ln_win, file_type):
    """
    Selects a spectral order for a line based and its bandpass.

    Parameters:
    -----------
    wave_2d : list of lists
        Wavelength in each spectral order [angstroms].
    ln_ctr : float
        Centre of the spectral line [angstroms].
    ln_win : float
        Bandwidth around the line where the flux is to be integrated
        [angstroms].

    Returns:
    --------
    order : {int, None}
        Spectral order for the given line parameters, None if no order is
        found.
    """
    min_wave = ln_ctr-ln_win/2.
    max_wave = ln_ctr+ln_win/2.

    ord = None
    orders = []
    for k in range(len(wave_2d)):
        if min_wave > wave_2d[k][0] and max_wave < wave_2d[k][-1]:
            ord = k
            orders.append(ord)

    print("Orders available =", orders)

    if file_type == "S2D":
        ord_type = "espresso"
    else:
        ord_type = "harps" #####

    print("ord_type =", ord_type)

    if ord_type == "harps":
        # Using two orders for the CaIIK line (as in Harps pipeline)
        if ln_ctr != 3933.664:
            orders = [orders[-1]]

    if ord_type == "espresso":
        # first two for CaIIK
        if ln_ctr == 3933.664:
            rders = orders[:-2]
        # last two for CaIIH
        if ln_ctr == 3968.47:
            orders = orders[2:]

    if ord_type == "last": # First actin version
        orders = [orders[-1]]

    if ord_type == "all": # All possible orders
        orders = orders

    print("Selected order =", orders)

    if ord is None:
        print("\n*** ERROR: Could not determine spectral order for:")
        print("*** min_wave = {:.2f}".format(min_wave))
        print("*** max_wave = {:.2f}".format(max_wave))

    return orders, ord_type


def line_plot(data, ord, ln_id, ln_ctr, ln_win, bandfunc, bandtype, out_dir):
    """
    Saves spectral lines plots.

    Parameters:
    -----------
    wave, flux : lists
        Wavelength [angstroms] and flux.
    obj : str
        Object identification.
    date : str
        Date of observations in fits file format.
    ln_id : str
        Spectral line identification.
    ln_ctr, ln_win : float
        Spectral line centre and bandpass [angstroms].
    bandfunc : list
        Bandpass function.
    file_type : str
        Identification of the type of file used. Options are 'S2D', 'S1D', 'e2ds', 's1d', 'ADP', and 'rdb'.
    out_dir : str
        Output directory to save the plots.

    Returns:
    --------
    Plots of the regions around the spectral lines used to calculate the
    indices. The plots are saved in the directory: 'out_dir'/'obj'.
    """

    print("Executing line_plot")

    obj       = data['obj']
    obs_date  = data['obs_date']
    file_type = data['file_type']
    instr     = data['instr']

    # 2d spectrum
    if isinstance(ord, int):
        wave = data['wave'][ord]
        flux = data['flux'][ord]/data['blaze'][ord]
        snr  = data['snr'][ord]
    # 1d spectrum
    elif ord is None:
        wave       = data['wave']
        flux       = data['flux']
        median_snr = data['median_snr']
    else:
        sys.exit("***ERROR: 'ord' must be either 'None' or instance of 'int'.")

    if bandtype == 'sq':
        wmin = ln_ctr - ln_win/2
        wmax = ln_ctr + ln_win/2
    if bandtype == 'tri':
        wmin = ln_ctr - ln_win
        wmax = ln_ctr + ln_win

    # Computing data with fractional pixels
    wave = wave[1:]
    flux = flux[1:]

    cond_reg = (wave > wmin - 4) & (wave < wmax + 4)
    flux_reg = flux[cond_reg]
    wave_reg = wave[cond_reg]

    cond = (wave > wmin) & (wave < wmax)

    dwave_l = wave[cond][0]-wmin
    dwave_r = wmax-wave[cond][-1]

    wave_win = np.insert(wave[cond], 0, wave[cond][0]-dwave_l)
    wave_win = np.append(wave_win, wave[cond][-1]+dwave_r)

    cond_l = (wave < wave[cond][0])
    cond_r = (wave > wave[cond][-1])

    flux_win = np.insert(flux[cond], 0, flux[cond_l][-1])
    flux_win = np.append(flux_win, flux[cond_r][0])

    flux_norm = (flux_reg-min(flux_reg))/(max(flux_reg)-min(flux_reg))

    wave_tot = np.r_[wave[cond_l], wave_win, wave[cond_r]]
    flux_tot = np.r_[flux[cond_l], flux_win, flux[cond_r]]

    if bandtype == 'sq':
        bandfunc_win = np.insert(bandfunc[cond], 0, 1.)
        bandfunc_win = np.append(bandfunc_win, 1.)
    if bandtype == 'tri':
        bandfunc_win = np.insert(bandfunc[cond], 0, 0.)
        bandfunc_win = np.append(bandfunc_win, 0.)

    band_l = np.zeros(len(wave[cond_l]))
    band_r = np.zeros(len(wave[cond_r]))

    width, height = ac_tools.plot_params()
    plt.figure(figsize=(width, height))

    plt.plot(wave_reg, flux_norm, color='k', ls='-', marker='', linewidth=1)
    #plt.axhline(0.0, c='k',ls=':')

    if bandtype == 'sq':
        bandfunc_tot = np.r_[band_l, 0., bandfunc_win[:-1], band_r]
        plt.plot(wave_tot, bandfunc_tot, color='r', ls='-', marker='', drawstyle='steps-pre')
    if bandtype == 'tri':
        bandfunc_tot = np.r_[band_l, bandfunc_win, band_r]
        plt.plot(wave_tot, bandfunc_tot, color='r', ls='-', marker='')

    plt.axvline(ln_ctr, color='r', ls=':', linewidth=1)
    #plt.axvline(wmin, color='r', ls='-', linewidth=1)
    #plt.axvline(wmax, color='r', ls='-', linewidth=1)

    plt.xlim(wave_reg[0], wave_reg[-1])
    plt.ylim(-0.1, 1.3)

    plt.ylabel('Normalized {} flux'.format(ln_id))
    plt.xlabel('Wavelength [Ang]')

    if file_type in ac_set.ftypes['1d']:
        plt.annotate("sp. median S/N = {:.2f}".format(median_snr),
                        xy=(0.05,0.9),
                        xycoords='axes fraction',
                        textcoords='axes fraction')

    if file_type in ac_set.ftypes['2d']:
        plt.annotate("S/N = {:.2f}".format(snr),
                        xy=(0.05,0.9),
                        xycoords='axes fraction',
                        textcoords='axes fraction')

    # For case of reading from rdb, obj and date will be huge repeating lists
    if type(obj) == list:
        obj = obj[0]
        obs_date = obs_date[0]

    # Save plots
    if out_dir and out_dir != 'show':
        print("Saving plot of line {}".format(ln_id))
        if ord is not None:
            save_name = '{}_{}_{}_{}_{}_ord{}_{}'.format(obj, obs_date, instr, file_type, ln_id, ord, ac_set.fnames['ln_plt'])
        else:
            save_name = '{}_{}_{}_{}_{}_{}'.format(obj, obs_date, instr, file_type, ln_id, ac_set.fnames['ln_plt'])

        # Create directories
        if not os.path.isdir(out_dir):
            os.makedirs(out_dir)
        if not os.path.isdir(os.path.join(out_dir, obj)):
            os.mkdir(os.path.join(out_dir, obj))

        plt.savefig(os.path.join(out_dir, obj, save_name))

    # Show plots
    if out_dir == 'show': plt.show()

    plt.close()

    return


def get_win(data, ln_id, ln_ctr, ln_win, bandtype, blaze=None, err=None, frac=True, ln_plts=False):
    """
    Calculate the flux for each line.
    """

    file_type  = data['file_type']
    wave       = data['wave']
    flux       = data['flux']
    obs_date   = data['obs_date']
    obj        = data['obj']
    snr        = data['snr']
    median_snr = data['median_snr']
    blaze      = data['blaze']
    noise      = data['noise']

    # 2d spectra
    if type(wave[0]) is np.ndarray:

        # NOTE: sel_ord is using ln_win/2 (as in sq bandpass)
        orders, ord_type = sel_order(wave, ln_ctr, ln_win, file_type)

        num     = []
        denum   = []
        snr_ord = []
        for k in range(len(orders)):
            ord = orders[k]
            print('Order = {}'.format(ord))
            print('SNR = {:.2f}'.format(snr[ord]))

            wave_ord  = wave[ord]
            flux_ord  = flux[ord]
            blaze_ord = blaze[ord]

            snr_ord.append(snr[ord])

            # Compute flux for line parameters
            f_sum, f_sum_var, bandfunc, npixels, flg, frac_neg = ac_tools.compute_flux(wave_ord, flux_ord, blaze_ord, noise, ln_ctr, ln_win, bandtype=bandtype, frac=frac)

            # Plot line regions
            if ln_plts:
                line_plot(data, ord, ln_id, ln_ctr, ln_win, bandfunc, bandtype, out_dir=ln_plts)

            num.append(f_sum/f_sum_var)
            denum.append(1.0/f_sum_var)

        # Weighted sum of fluxes in the orders
        f_sum = sum(num)/sum(denum)

        # Compute error
        f_sum_var = 1.0/sum(denum)
        if f_sum_var >= 0.0:
            f_sum_err = np.sqrt(f_sum_var)
        else:
            print("*** ERROR: Variance of flux in the line is negative. Cannot compute error.")
            f_sum_err = 0.0

        # mMdian snr of the used orders
        snr = np.median(snr_ord)

    # 1d spectra
    elif type(wave[0]) in [np.float, np.float64]:
        ord      = None
        ord_type = None
        wave     = wave
        flux     = flux

        # Compute flux for line parameters
        f_sum, f_sum_var, bandfunc, npixels, flg, frac_neg = ac_tools.compute_flux(wave, flux, blaze, noise, ln_ctr, ln_win, bandtype=bandtype, frac=frac)

        # Compute error
        if f_sum_var >= 0.0:
            f_sum_err = np.sqrt(f_sum_var)
        else:
            print("*** ERROR: Variance of flux in the line is negative. Cannot compute error.")
            f_sum_err = 0.0

        # Plot line regions
        if ln_plts:
            line_plot(data, ord, ln_id, ln_ctr, ln_win, bandfunc, bandtype, out_dir=ln_plts)

    win = {}
    win['sum']      = f_sum
    win['sum_err']  = f_sum_err
    win['flg']      = flg
    win['frac_neg'] = frac_neg
    win['snr']      = snr
    win['order']    = ord
    win['bandfunc'] = bandfunc
    win['ord_type'] = ord_type
    win['npixels']  = npixels

    return win



#!/usr/bin/env python

# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import sys, os
import pylab as plt
import numpy as np

import datetime

# ACTIN FILES:
import ac_tools
import ac_settings as ac_set


def check_duplicate(obj, date, instr, file_type, out_dir):
    """
    Check if measurement is a duplicate in the output file.

    Parameters:
    -----------
    obj : str
        Object identification.
    date : str
        Date of observation in the fits file format.
    file_type : str
        Type of file used: 'S2D', 'S1D', 'e2ds', 's1d', 'ADP', or 'rdb'.
    output_dir : str
        Directory of output file.

    Returns:
    --------
    bool
        True if duplicate, False otherwise.
    """

    print()
    print("Executing check_duplicate:")

    if obj is None or date is None or out_dir is None: return

    file_name = "{}_{}_{}_{}".format(obj, instr, file_type, ac_set.fnames['data'])
    pfile_name = os.path.join(out_dir, obj, file_name)

    if os.path.isfile(pfile_name):
        try:
            rdb_data = ac_tools.read_rdb(pfile_name)[0]
        except:
            print("*** ERROR: Cannot read rdb file:")
            print(pfile_name)
            sys.exit()
        if date in rdb_data['obs_date']:
            print("Date {} already saved in:".format(date))
            print(pfile_name)
            print("*** ACTION: Ignoring measurement")
            return True
        else:
            print("Date {} not present in {}".format(date, pfile_name))
    else: print("No data saved for {}, {}.".format(obj, date))

    return False


def save_data(data, index, out_dir):
    """
    Saves output data to rdb file.
    For a description of the returned headers see README.md file.
    """

    print()
    print("SAVING DATA")
    print("-----------")

    print("Executing save_data")

    if data is None or out_dir is None:
        print("*** ERROR: 'data' is None or 'out_dir' is None")
        return

    # reading from rdb file
    if data['file_type'] == 'rdb':
        data_keys = ['obj', 'obs_date', 'bjd']
    else:
        data_keys = ac_set.outkeys

    if not os.path.isdir(out_dir):
        os.makedirs(out_dir)
    if not os.path.isdir(os.path.join(out_dir, data['obj'])):
        os.mkdir(os.path.join(out_dir, data['obj']))

    # Include index information
    if index is not None:
        # convert index data from table list to dictionary
        indices = [index['index'][k] for k in range(len(index['index']))]
        ind = {}
        for k in range(len(indices)):
            ind[indices[k]] = index['value'][k]
            ind['{}_err'.format(indices[k])] = index['error'][k]
            ind['{}_snr'.format(indices[k])] = index['snr'][k]
            ind['{}_flg'.format(indices[k])] = index['flg'][k]
            ind['{}_mfrac_neg'.format(indices[k])] = index['mfrac_neg'][k]

        # For npixels in each line
        index_keys = list(index)
        for k in range(len(index_keys)):
            if index_keys[k].split('_')[-1] == 'npixels':
                ind['{}_npixels'.format(index_keys[k].split('_')[0])] = index['{}_npixels'.format(index_keys[k].split('_')[0])]

        #index_keys = list(ind.keys())
        index_keys = list(ind)
        index_keys.sort()

        keys = data_keys + index_keys

        # merging data and index dictionaries to use below
        all_data = dict(data)
        all_data.update(ind)
    else:
        keys = data_keys
        all_data = data

    # for save_rdb to work need to have the dic with lists
    data_save = {}
    for k in range(len(keys)):
        data_save[keys[k]] = [all_data[keys[k]]]

    file_data = "{}_{}_{}_{}".format(all_data['obj'], data['instr'], data['file_type'], ac_set.fnames['data'])
    pfile_data = os.path.join(out_dir, all_data['obj'], file_data)

    if not os.path.isfile(pfile_data):
        ac_tools.save_rdb(data_save, keys, pfile_data)
        print("Data saved to:\n{}".format(pfile_data))
    else:
        ac_tools.add_rdb(data_save, keys, pfile_data)
        print("Data added to:\n{}".format(pfile_data))

    return pfile_data


def save_log(info, options, n_files, out_dir):

    if out_dir is False: return

    print()
    print("Executing save_log:")

    file_type = info['file_type']
    instr = info['instr']
    files = info['source_path']
    obj = info['obj']

    file_log = "{}_{}_{}_{}".format(obj, instr, file_type, ac_set.fnames['log_data'])
    pfile_log = os.path.join(out_dir, obj, file_log)

    date_now = str(datetime.datetime.now()).split(".")[0]

    if type(files) is list and len(files) > 1:
        source_path = os.path.commonprefix(files)
    else: source_path = os.path.split(files)[0]

    log = {}
    log['version'] = info['version']
    log['run_date'] = date_now
    log['source_path'] = info['source_path']
    log['tel'] = info['tel']
    log['instr'] = info['instr']
    log['file_type'] = info['file_type']
    log['n_files'] = n_files
    log['frac'] = options['frac']

    with open(pfile_log,'w') as log_file:
        for k in range(len(list(log))):
            print("{}={}".format(list(log)[k],log[list(log)[k]]), file=log_file)

    print("Log saved to:\n{}".format(pfile_log))

    return pfile_log


def save_line_info(info, sel_lines, out_dir):

    if out_dir is False: return
    if not sel_lines: return

    print()
    print("Executing save_line_info:")

    obj = info['obj']
    file_type = info['file_type']
    instr = info['instr']

    keys = ["ind_id", "ind_var", "ln_id", "ln_c", "ln_ctr", "ln_win", "bandtype"]

    file_lines = "{}_{}_{}_{}".format(obj, instr, file_type, ac_set.fnames['lines_data'])
    pfile_lines = os.path.join(out_dir, obj, file_lines)

    ac_tools.save_rdb(sel_lines, keys, pfile_lines)
    print("Lines information saved to:\n{}".format(pfile_lines))

    return pfile_lines



#!/usr/bin/env python

import os
import subprocess

try: from setuptools import setup
except: from distutils.core import setup


path = os.path.dirname(os.path.realpath(__file__))
version_file = os.path.join(path, "actin", "VERSION")

try:
    with open(version_file, 'r') as file:
        version = file.read()
except: version = "unknown"


setup(name = 'actin',
      version = version,
      description = 'Activity Indices Calculator',
      url = 'http://github.com/gomesdasilva/ACTIN',
      download_url = 'https://github.com/gomesdasilva/ACTIN/archive/v1.3.2.tar.gz',
      author = 'Joao Gomes da Silva',
      author_email = 'Joao.Silva@astro.up.pt',
      license = 'MIT',
      keywords = ['astronomy', 'activity', 'fits', 'harps', 'harps-n', 'espresso','radial velocity', 'exoplanets'],
      packages = ['actin'],
      entry_points = {
        "console_scripts": ['actin = actin.actin:main']
        },
      include_package_data = True,
      install_requires = ['appdirs']
      )


# This runs ACTIN from terminal, creates ACTIN directory with config file in Application Support and gives its location
#try: subprocess.call(["actin"])
#except: pass



#!/usr/bin/env python

# compatibility with python 2/3:
from __future__ import print_function
from __future__ import division

import numpy as np


def init():

    # Accepted file types
    global ftypes
    ftypes = {}
    ftypes['1d'] = ['S1D',
                    's1d',
                    'ADP',
                    'rdb']

    ftypes['2d'] = ['S2D',
                    'e2ds']

    ftypes['all'] = ftypes['1d'] + ftypes['2d']

    # Accepted instruments
    global instr
    instr =    ['HARPS',
                'HARPN',
                'ESPRESSO']

    # File names
    global fnames
    fnames = {}
    fnames['data'] = "data.rdb"
    fnames['log_data'] = "log.txt"
    fnames['lines_data'] = "lines.txt"
    fnames['ln_plt'] = ".pdf"
    fnames['time_plt'] = "time.pdf"
    fnames['time_mlty_plt'] = "time_mlty.pdf"

    # output keys (used in ac_seve.py)
    global outkeys
    outkeys = {}
    outkeys = ['obj', 'instr', 'obs_date', 'bjd', 'rv', 'rv_err', 'fwhm', 'fwhm_err', 'cont', 'cont_err', 'bis', 'bis_err', 'noise', 'median_snr', 'data_flg', 'bv', 'airmass_end']

    # err messages
    global err
    err = {}
    err['ERROR'] = "*** ERROR:"


def preamble(version_file):

    __author__ = "Joao Gomes da Silva"

    try:
        with open(version_file, 'r') as file:
            version = file.read().splitlines()[0]
        print("\nACTIN {}".format(version))
    except:
        print("*** WARNING | Unable to read 'VERSION' file.")
        version = "Unknown"

    print("Instituto de Astrofisica e Ciencias do Espaco")
    print("Centro de Astrofisica da Universidade do Porto")
    print("Author:", __author__+ ",", "Joao.Silva@astro.up.pt")

    return version
